"""Test result parsers for JUnit XML and Pytest JSON formats."""

from elspais.graph.parsers.results.junit_xml import JUnitXMLParser
from elspais.graph.parsers.results.pytest_json import PytestJSONParser

__all__ = ["JUnitXMLParser", "PytestJSONParser"]
